@@ -13,16 +13,17 @@ import mpmath
 
 
 # Add any Sphinx extension module names here, as strings.
-extensions = ['sphinx.ext.autodoc', 'sphinx.ext.mathjax']
+extensions = ['sphinx.ext.autodoc', 'sphinx.ext.mathjax',
+              'sphinx.ext.intersphinx', 'sphinxcontrib.autoprogram',
+              'matplotlib.sphinxext.plot_directive']
 
-# The master toctree document.
-master_doc = 'index'
+# Sphinx will warn about all references where the target cannot be found.
+nitpicky = True
 
 # Project information.
-project = 'mpmath'
-copyright = '2007-2021, Fredrik Johansson and mpmath developers'
-version = mpmath.__version__
-release = version
+project = mpmath.__name__
+copyright = '2007-2026, Fredrik Johansson and mpmath developers'
+release = version = mpmath.__version__
 
 # Define how the current time is formatted using time.strftime().
 today_fmt = '%B %d, %Y'
@@ -30,16 +31,22 @@ today_fmt = '%B %d, %Y'
 # The "theme" that the HTML output should use.
 html_theme = 'classic'
 
-# Output file base name for HTML help builder.
-htmlhelp_basename = 'mpmathdoc'
-
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, document class [howto/manual]).
-latex_documents = [(master_doc, 'mpmath.tex', 'mpmath documentation',
+latex_documents = [('index', 'mpmath.tex', 'mpmath documentation',
                     r'Fredrik Johansson \and mpmath contributors', 'manual')]
 
-# Additional stuff for the LaTeX preamble.
-latex_preamble = r'\usepackage{amsfonts}'
-
 # The name of default reST role, that is, for text marked up `like this`.
 default_role = 'math'
+
+# Contains mapping the locations and names of other projects that
+# should be linked to in this documentation.
+intersphinx_mapping = {
+    'python': ('https://docs.python.org/3/', None),
+    'sympy': ('https://docs.sympy.org/latest/', None),
+}
+
+plot_include_source = True
+plot_formats = [('png', 96), 'pdf']
+plot_html_show_formats = False
+plot_html_show_source_link = False
